local addonName, addon = ...;

-- luacheck: globals InspectFrame InspectPaperDollItemsFrame CharacterFrame C_PaperDollInfo

local oPrint = print;
local function print(...)
	if(true)then
		local msg = strjoin(" ",tostringall(...));
		oPrint("|cff6600ccBetterCharacterPanel|r:",msg);
	end
end

local buttonLayout =
{
	[1]	= "left",
	[2]	= "left",
	[3]	= "left",
	[15] = "left",
	[5]	= "left",
	[9]	= "left",

	[10] = "right",
	[6] = "right",
	[7] = "right",
	[8] = "right",
	[11] = "right",
	[12] = "right",
	[13] = "right",
	[14] = "right",

	[16] = "center",
	[17] = "center",
};

local function ColorGradient(perc, ...)
	if perc >= 1 then
		local r, g, b = select(select('#', ...) - 2, ...);
		return r, g, b;
	elseif perc <= 0 then
		local r, g, b = ...;
		return r, g, b;
	end

	local num = select('#', ...) / 3;

	local segment, relperc = math.modf(perc*(num-1));
	local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...);

	return r1 + (r2-r1)*relperc, g1 + (g2-g1)*relperc, b1 + (b2-b1)*relperc;
end

local function ColorGradientHP(perc)
	return ColorGradient(perc,1,0,0, 1,1,0, 0,1,0);
end

local enchantReplacementTable =
{
	["Stamina"] = "Stam",
	["Intellect"] = "Int",
	["Agility"] = "Agi",
	["Strength"] = "Str",

	["Mastery"] = "Mast",
	["Versatility"] = "Vers",
	["Critical Strike"] = "Crit",
	["Haste"] = "Haste",

	["Minor Speed Increase"] = "Speed",
};

local function ProcessEnchantText(enchantText)
	for seek,replacement in pairs(enchantReplacementTable) do
		enchantText = enchantText:gsub(seek,replacement);
	end
	return enchantText;
end

local scanningTooltip = CreateFrame("GameTooltip", "BCPScanningTooltip", nil, "GameTooltipTemplate");

local function GetItemEnchatAsText(itemLink)
	scanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
	scanningTooltip:SetHyperlink(itemLink);

	for i=1,scanningTooltip:NumLines() do
		local text = _G["BCPScanningTooltipTextLeft"..i]:GetText();
		local enchantText = string.match(text,"Enchanted: (.*)");
		if(enchantText)then
			return ProcessEnchantText(enchantText);
		end
	end

	return  "";
end

local function GetSocketTextures(itemLink)
	scanningTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
	scanningTooltip:SetHyperlink(itemLink);

	local textures = {};
	for i=1,10 do
		local textureFrame = _G["BCPScanningTooltipTexture"..i];
		local texture = textureFrame and textureFrame:IsShown() and textureFrame:GetTexture();
		if(texture)then
			table.insert(textures,texture);
		end
	end

	return textures;
end

local function AnchorTextureLeftOfParent(parent,textures)
	textures[1]:SetPoint("RIGHT",parent,"LEFT",-3,1);
	for i=2,4 do
		textures[i]:SetPoint("RIGHT",textures[i - 1],"LEFT",-2,0);
	end
end

local function AnchorTextureRightOfParent(parent,textures)
	textures[1]:SetPoint("LEFT",parent,"RIGHT",3,1);
	for i=2,4 do
		textures[i]:SetPoint("LEFT",textures[i - 1],"RIGHT",2,0);
	end
end

local function CreateAdditionalDisplayForButton(button)
	local parent = button:GetParent();
	local additionalFrame = CreateFrame("frame",nil,parent);
	additionalFrame:SetWidth(100);

	additionalFrame.ilvlDisplay = additionalFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightOutline");

	additionalFrame.enchantDisplay = additionalFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightOutline");
	additionalFrame.enchantDisplay:SetTextColor(0,1,0,1);

	additionalFrame.durabilityDisplay = CreateFrame("StatusBar", nil, additionalFrame);
	additionalFrame.durabilityDisplay:SetMinMaxValues(0,1);
	additionalFrame.durabilityDisplay:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar");
	additionalFrame.durabilityDisplay:GetStatusBarTexture():SetHorizTile(false);
	additionalFrame.durabilityDisplay:GetStatusBarTexture():SetVertTile(false);
	additionalFrame.durabilityDisplay:SetHeight(40);
	additionalFrame.durabilityDisplay:SetWidth(2.3);
	additionalFrame.durabilityDisplay:SetOrientation("VERTICAL");

	additionalFrame.socketDisplay = {};

	for i=1,4 do
		additionalFrame.socketDisplay[i] = additionalFrame:CreateTexture();
		additionalFrame.socketDisplay[i]:SetWidth(14);
		additionalFrame.socketDisplay[i]:SetHeight(14);
	end

	return additionalFrame;
end

local function positonLeft(button)
	local additionalFrame = button.BCPDisplay;

	additionalFrame:SetPoint("TOPLEFT",button,"TOPRIGHT");
	additionalFrame:SetPoint("BOTTOMLEFT",button,"BOTTOMRIGHT");

	additionalFrame.ilvlDisplay:SetPoint("BOTTOMLEFT",additionalFrame,"BOTTOMLEFT",10,2);
	additionalFrame.enchantDisplay:SetPoint("TOPLEFT",additionalFrame,"TOPLEFT",10,-7);

	additionalFrame.durabilityDisplay:SetPoint("TOPLEFT",button,"TOPLEFT",-6,0);
	additionalFrame.durabilityDisplay:SetPoint("BOTTOMLEFT",button,"BOTTOMLEFT",-6,0);

	AnchorTextureRightOfParent(additionalFrame.ilvlDisplay,additionalFrame.socketDisplay);
end

local function positonRight(button)
	local additionalFrame = button.BCPDisplay;

	additionalFrame:SetPoint("TOPRIGHT",button,"TOPLEFT");
	additionalFrame:SetPoint("BOTTOMRIGHT",button,"BOTTOMLEFT");

	additionalFrame.ilvlDisplay:SetPoint("BOTTOMRIGHT",additionalFrame,"BOTTOMRIGHT",-10,2);
	additionalFrame.enchantDisplay:SetPoint("TOPRIGHT",additionalFrame,"TOPRIGHT",-10,-7);

	additionalFrame.durabilityDisplay:SetWidth(1.2);
	additionalFrame.durabilityDisplay:SetPoint("TOPRIGHT",button,"TOPRIGHT",4,0);
	additionalFrame.durabilityDisplay:SetPoint("BOTTOMRIGHT",button,"BOTTOMRIGHT",4,0);

	AnchorTextureLeftOfParent(additionalFrame.ilvlDisplay,additionalFrame.socketDisplay);
end

local function positonCenter(button)
	local additionalFrame = button.BCPDisplay;

	additionalFrame:SetPoint("BOTTOMLEFT",button,"BOTTOMLEFT",-100,0);
	additionalFrame:SetPoint("TOPRIGHT",button,"TOPRIGHT",0,-100);

	additionalFrame.durabilityDisplay:SetHeight(2);
	additionalFrame.durabilityDisplay:SetWidth(40);
	additionalFrame.durabilityDisplay:SetOrientation("HORIZONTAL");
	additionalFrame.durabilityDisplay:SetPoint("BOTTOMLEFT",button,"BOTTOMLEFT",0,-2);
	additionalFrame.durabilityDisplay:SetPoint("BOTTOMRIGHT",button,"BOTTOMRIGHT",0,-2);

	additionalFrame.ilvlDisplay:SetPoint("BOTTOM",button,"TOP",0,7);

--left center
	if(button:GetID() == 16)then
		additionalFrame.enchantDisplay:SetPoint("BOTTOMRIGHT",button,"BOTTOMLEFT",-5,0);

		AnchorTextureLeftOfParent(additionalFrame.ilvlDisplay,additionalFrame.socketDisplay);
	else
		additionalFrame.enchantDisplay:SetPoint("BOTTOMLEFT",button,"BOTTOMRIGHT",5,0);

		AnchorTextureRightOfParent(additionalFrame.ilvlDisplay,additionalFrame.socketDisplay);
	end
end

local function AnchorAdditionalDisplay(button)
	local layout = buttonLayout[button:GetID()];
	if(layout == "left")then
		positonLeft(button);
	elseif(layout == "right")then
		positonRight(button)
	elseif(layout == "center")then
		positonCenter(button)
	end
end

local function UpdateAdditionalDisplay(button,unit)
	local additionalFrame = button.BCPDisplay;
	local itemLink = GetInventoryItemLink(unit,button:GetID());

	local itemiLvlText = "";
	if(itemLink)then
		local ilvl = GetDetailedItemLevelInfo(itemLink);
		local quality = GetInventoryItemQuality(unit, button:GetID());
		local hex = select(4,GetItemQualityColor(quality));
		itemiLvlText = "|c"..hex..ilvl.."|r";
	end
	additionalFrame.ilvlDisplay:SetText(itemiLvlText);

	local enchantText = itemLink and GetItemEnchatAsText(itemLink) or "";
	additionalFrame.enchantDisplay:SetText(string.sub(enchantText,0,18));

	local textures = itemLink and GetSocketTextures(itemLink) or {};
	for i=1,4 do
		if(#textures >= i)then
			additionalFrame.socketDisplay[i]:SetTexture(textures[i]);
			additionalFrame.socketDisplay[i]:Show();
		else
			additionalFrame.socketDisplay[i]:Hide();
		end
	end

	local currentDurablity, maxDurability = GetInventoryItemDurability(button:GetID());
	local percDurability = currentDurablity and currentDurablity/maxDurability;
	if(UnitIsUnit("player",unit) and percDurability and percDurability < 1)then
		additionalFrame.durabilityDisplay:Show();
		additionalFrame.durabilityDisplay:SetValue(percDurability);
		additionalFrame.durabilityDisplay:SetStatusBarColor(ColorGradientHP(percDurability));
	else
		additionalFrame.durabilityDisplay:Hide();
	end
end

local function CreateInspectIlvlDisplay()
	local parent = InspectPaperDollItemsFrame;
	if(not parent.ilvlDisplay)then
		parent.ilvlDisplay = parent:CreateFontString(nil, "OVERLAY", "GameFontHighlightOutline22");
		parent.ilvlDisplay:SetPoint("TOPRIGHT",parent,"TOPRIGHT",0,-20);
		parent.ilvlDisplay:SetPoint("BOTTOMLEFT",parent,"TOPRIGHT",-80,-67);
	end
end

local function UpdateInspectIlvlDisplay(unit)
	local ilvl = C_PaperDollInfo.GetInspectItemLevel(InspectFrame.unit);
	local color;
	if(ilvl <= 140)then
		color = "1eff00";
	elseif(ilvl <= 180)then
		color = "0070dd";
	elseif(ilvl <= 220)then
		color = "a335ee";
	elseif(ilvl <= 240)then
		color = "ff8000";
	else
		color = "e6cc80";
	end

	local parent = InspectPaperDollItemsFrame;
	parent.ilvlDisplay:SetText(string.format("|cff%s%d|r",color,ilvl));
end

hooksecurefunc("PaperDollItemSlotButton_Update",function(button)
	if(not buttonLayout[button:GetID()])then return; end

	if(not button.BCPDisplay)then
		button.BCPDisplay = CreateAdditionalDisplayForButton(button);
		AnchorAdditionalDisplay(button);
	end

	UpdateAdditionalDisplay(button,"player");
end);

function addon:ADDON_LOADED(addonName)
	if(addonName == "Blizzard_InspectUI")then
		hooksecurefunc("InspectPaperDollItemSlotButton_Update",function(button)
			if(not button.BCPDisplay)then
				button.BCPDisplay = CreateAdditionalDisplayForButton(button);
				AnchorAdditionalDisplay(button);
			end
			UpdateAdditionalDisplay(button,InspectFrame.unit);
		end);

		hooksecurefunc("InspectPaperDollFrame_SetLevel",function()
			if(not InspectFrame.unit)then return; end
			CreateInspectIlvlDisplay();
			UpdateInspectIlvlDisplay(InspectFrame.unit);
		end);
	end
end

local characterSlots = {
	"CharacterHeadSlot",
	"CharacterNeckSlot",
	"CharacterShoulderSlot",
	"CharacterChestSlot",
	"CharacterWaistSlot",
	"CharacterLegsSlot",
	"CharacterFeetSlot",
	"CharacterWristSlot",
	"CharacterHandsSlot",
	"CharacterFinger0Slot",
	"CharacterFinger1Slot",
	"CharacterTrinket0Slot",
	"CharacterTrinket1Slot",
	"CharacterBackSlot",
	"CharacterMainHandSlot",
	"CharacterSecondaryHandSlot",
};

local function updateAllCharacterSlots()
	for _,slot in ipairs(characterSlots) do
		local button = _G[slot];
		if(button)then
			UpdateAdditionalDisplay(button,"player");
		end
	end
end

local lastUpdate = 0;
function addon:SOCKET_INFO_UPDATE()
	if(CharacterFrame:IsShown())then
		local time = GetTime();
		if(time ~= lastUpdate)then
			updateAllCharacterSlots();
			lastUpdate = time;
		end
	end
end

local eventListener = CreateFrame("frame");
eventListener:SetScript("OnEvent",function (self,event,...)
	addon[event](addon,...);
end);
eventListener:RegisterEvent("ADDON_LOADED");
eventListener:RegisterEvent("SOCKET_INFO_UPDATE");